<?php
namespace Engine\Debug\ParserCallTrace;

class ParserCallTraceDummy
{
	/**
	 * Does nothing
	 */
	public function reset()
	{
	}

	public function destroy()
	{
	}

	public function setMaxStringLength($maxStringLength)
	{
		return $this;
	}

	public function addClassHandler($callback)
	{
		return $this;
	}

	public function addMethodHandler($callback)
	{
		return $this;
	}
	public function addDataHandler($callback)
	{
		return $this;
	}

	public function logCallStart($obj, $mainMethod, $args)
	{
		return $this;
	}

	public function logCallEnd()
	{
		return $this;
	}

	public function logInput($args, $style='default')
	{
		return $this;
	}

	public function logInputWithNull($args, $style='default')
	{
		return $this;
	}

	public function logOutput($args, $style='default')
	{
		return $this;
	}

	public function logOutputWithNull($args, $style='default')
	{
		return $this;
	}

	public function logStep($args, $style='default')
	{
		return $this;
	}

	public function logSeparator($style='default')
	{
		return $this;
	}

	public function dlog($text, $args = [])
	{
		return $this;
	}
}
